from jaanca_utils_os.utils.helpers.parse_bool import parse_bool
import ast
import json

def is_float(value:str):
    try:
        float(value)
        return True
    except:
        return False

def is_int(value:str):
    try:
        int(value)
        return True
    except:
        return False

def is_list(value:str):
    try:
        if isinstance(ast.literal_eval(value),list):
            return True
        else:
            return False
    except:
        return False

def is_dict(value:str):
    try:
        json_data = json.loads(value)
        dict(json_data)
        return True
    except:
        return False

def parse_types(data):
    '''Description
    :return: the conversion or cast (int)(dict) of the identified data type of the received parameter
    '''
    parse_data=parse_bool(data)
    if isinstance(parse_data,bool):
        return parse_data
    elif is_int(data):
        return int(data)
    elif is_float(data):
        return float(data)
    elif is_list(data):
        return list(ast.literal_eval(data))
    elif is_dict(data):
        json_data = json.loads(data)
        return dict(json_data)
    else:
        return data



from enum import StrEnum
import datetime
import os


class FilePropertiesErrorCodes(StrEnum):
    ATTRIBUTES_READING_ERROR = "File not found error or there is an error reading the file attributes"

class FileProperties:
    """File Properties.
    Functionalities to read properties of an operating system file, such as: modification date, creation, size in bytes, among others.
    
    ## Example

    ```Python

    from jaanca_utils_os import FileProperties, FileFolderManagement
    import json

    file_name="hello asa s sas. as as a. as as .txt"
    filename_full_path_from_current_folder=FileFolderManagement.build_full_path_from_current_folder(__file__,folder_list=["file"])
    file_properties=FileProperties(filename_full_path_from_current_folder)
    status = file_properties.get_attribute_reading_status()
    if status is True:
        print(json.dumps(file_properties.get_dict(),indent=4))
        print(f"name:{file_properties.name}")
        print(f"extension:{file_properties.extension}")
        print(f"modification_date:{file_properties.modification_date}")
    else:
        print(status)        
    ```
    
    """
    name:str
    """File name."""
    extension:str
    """Filename extension."""
    modification_date:str
    """File modification date and time."""
    creation_date:str
    """File creation date and time."""
    access_date:str
    """Date and time of last access to the file."""
    size:int
    """The size of the content file. The length of blob in bytes."""
    date_format:str
    """date format used to present information."""

    def __init__(self,full_path_name:str) -> None:
        if(os.path.exists(full_path_name)):
            self.full_path_name=full_path_name
            self.date_format="%Y-%m-%d %H:%M:%S"
            self.__get_name_properties()
            self.__get_modification_date()
            self.__get_creation_date()
            self.__get_access_date()
            self.__get_size()
        else:
            self.__dict__ = {}

    def __get_name_properties(self)->str:
        self.name = os.path.basename(self.full_path_name)
        item_list=self.name.split(".")        
        self.extension = item_list[len(item_list)-1]

    def __get_modification_date(self)->str:
        modification_time = os.path.getmtime(self.full_path_name)
        self.modification_date = datetime.datetime.fromtimestamp(modification_time).strftime(self.date_format)

    def __get_creation_date(self)->str:
        creation_time = os.path.getctime(self.full_path_name)
        self.creation_date = datetime.datetime.fromtimestamp(creation_time).strftime(self.date_format)

    def __get_access_date(self)->str:
        access_time = os.path.getatime(self.full_path_name)
        self.access_date = datetime.datetime.fromtimestamp(access_time).strftime(self.date_format)

    def __get_size(self):
        self.size = os.path.getsize(self.full_path_name)
        
    def get_attribute_reading_status(self)->bool|str:
        if(len(self.__dict__)==0):
            return FilePropertiesErrorCodes.ATTRIBUTES_READING_ERROR
        else:
            return True        

    def get_dict(self)->dict:
        return self.__dict__
        




from enum import StrEnum
from pathlib import Path
from jaanca_utils_os.utils.helpers.parse_types import parse_types
import os

class EnvironmentParserLoaderErrorCodes:
    class KeyErrors(StrEnum):
        VARIABLES_HAVE_NOT_BEEN_CREATED = "The following environment variables have not been created or assigned a default value: {}"
    class Dependencies(StrEnum):
        DOTENV = "I need the python-dotenv>=1.0.0 library to work."

try:
    from dotenv import load_dotenv
except Exception as e:
    print(EnvironmentParserLoaderErrorCodes.Dependencies.DOTENV)

class EnvironmentParserLoader:
    '''Description
    A class that loads, parses, and manages system environment variables into various data types.
    General Functionality

    ## The EnvironmentParserLoader class would be designed to:
    - Read the system’s environment variables.
    - Parse (convert) these variables into specific data types (such as int, float, bool, str, list, dict).
    - Manage and provide centralized methods to access these environment variables in a typed and secure manner.
    - If the environment variable does not exist or has a value of None and a default value is not assigned, a KeyError exception will be returned.

    ## Considerations on environmental variables
    - An object must be specified with the variables that you want to create.
    - The attributes of these objects will be loaded with the environment variables, according to the name assigned to them.
    - If the attribute is mandatory, it will only be of type str and will contain the name of the environment variable to read.
    - If the environment variable is optional or may not exist, the attribute must be of type tuple or list, where the first element is the environment variable to read and the second the default value to assign.

    Example:
    class Environment:
        ENV_VAR_NO_EXIT = ("VARIABLE","Not Exist")
        ENV_VAR_IS_MANDATORY = "VARIABLE"    
    
    ### Example: Prerequisites
    ```console    
    pip install prettytable==3.10.0
    pip install python-dotenv==1.0.1
    ```

    ### File with environments vars .env
    ```console
    ENGINE_POSTGRES_CONN_HOST=psqlt
    ENGINE_POSTGRES_CONN_DB=test
    ENGINE_POSTGRES_CONN_PASSWORD=es3bv3v3
    ENGINE_POSTGRES_CONN_PORT=5432
    ENGINE_POSTGRES_CONN_USER=postgres
    ENGINE_POSTGRES_CONN_SSL=false
    FLOAT=3.3
    LIST=[1,2,3,"4","5"]
    DICT='{"one": "one", "two": 2}'
    BOOL_TRUE = true
    BOOL_TRUE_ONE = 1
    BOOL_TRUE_TWO = "1"
    BOOL_FALSE_ONE = 0
    BOOL_FALSE_TWO = "0"
    BOOL_FALSE_INCORRECT = "incorrect"
    ```
    
    ### Example
    ```Python
    from jaanca_utils_os import EnvironmentParserLoader, FileFolderManagement
    from prettytable import PrettyTable

    class Environment:
        HOST = "ENGINE_POSTGRES_CONN_HOST"
        DB_NAME = "ENGINE_POSTGRES_CONN_DB"
        PASSWORD = "ENGINE_POSTGRES_CONN_PASSWORD"
        PORT = "ENGINE_POSTGRES_CONN_PORT"
        USER = "ENGINE_POSTGRES_CONN_USER"
        SSL = "ENGINE_POSTGRES_CONN_SSL"
        FLOAT = "FLOAT"
        LIST = "LIST"
        DICT = "DICT"
        BOOL_TRUE = "BOOL_TRUE"
        BOOL_TRUE_ONE = "BOOL_TRUE_ONE"
        BOOL_TRUE_TWO = "BOOL_TRUE_TWO"
        BOOL_FALSE_ONE = "BOOL_FALSE_ONE"
        BOOL_FALSE_TWO = "BOOL_FALSE_TWO"
        BOOL_FALSE_INCORRECT = "BOOL_FALSE_INCORRECT"
        NO_DATA_TUPLE = ("VARIABLE","Not Exist")
        NO_DATA_LIST = ["VARIABLE","Not Exist"]
        NO_DATA_BOOL = ["VARIABLE","1"]

    ############
    # select the location of the file to read
    ############

    ############
    # Option 1
    # Run the script from where the default environment variables .env file is located
    # settings:Environment = EnvironmentParserLoader(Environment)

    ############
    # Other Options
    # Load varibles from current folder and subfolders
    env_full_path = FileFolderManagement.build_full_path_from_current_folder(__file__,filename=".env",folder_list=["folder2"])
    # Load varibles from disk path: c:\tmp
    env_full_path = FileFolderManagement.build_full_path_to_file("c:",file_name=".env",folder_list=["tmp"])
    # Load varibles from current folder
    env_full_path = FileFolderManagement.build_full_path_from_current_folder(__file__,filename=".env")

    settings:Environment = EnvironmentParserLoader(Environment,env_full_path=env_full_path)

    def print_attributes(cls):
        columns = ["Name", "Type", "Value"]
        myTable = PrettyTable(columns)
        for attribute_name, attribute_value in vars(cls).items():
            attribute_type = type(attribute_value)
            myTable.add_row([attribute_name, attribute_type.__name__, attribute_value])
        print(myTable)        

    print_attributes(settings)

    ```
    '''
    def __init__(self,cls,env_full_path:str=''):
        if env_full_path == '':
            env_full_path = Path('.') / '.env'
        load_dotenv(dotenv_path=env_full_path)
        for key, value in vars(cls).items():
            if not key.startswith('__'):
                default_value = None
                if isinstance(value,tuple) or isinstance(value,list):
                    default_value=value[1]
                    value=value[0]
                env_data=parse_types(os.getenv(value,default_value))
                setattr(self, key, env_data)

        self.mandatory_attribute_validation()

    def mandatory_attribute_validation(self):
        attributes = vars(self)
        none_attributes = [attr for attr, value in attributes.items() if value is None]
        if len(none_attributes)!=0:
            raise KeyError(EnvironmentParserLoaderErrorCodes.KeyErrors.VARIABLES_HAVE_NOT_BEEN_CREATED.format(none_attributes))

